/*
 *  MOB: A Memory Organization Benchmarking Tool 
 *
 *  Copyright (C) 2000-2001 Josep M. Blanquer and Robert C. Chalmers 
 *
 *  This package is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Josep M. Blanquer <blanquer@cs.ucsb.edu> 
 *          Robert C. Chalmers <robertc@cs.ucsb.edu> 
 *
 */
/**
 * Code generation debug program.
 **/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/time.h>
#include "inc/mob.h"
#include "inc/genCode.h"

#if 0
#define MACH_TIMERS
#undef REP_BLOCK
#endif

#define MIN_SETS  100
#define MILL      1000000L


#if defined(MACH_i386) && defined(MACH_TIMERS)
/* profile register timers in i386 */
inline long long rdtsc(void) {
  
  unsigned int tmp[2];
  
  __asm__ ("rdtsc"
	   : "=a" (tmp[1]), "=d" (tmp[0])
	   : "c" (0x10) );
  
  return ( ((long long)tmp[0] << 32 | tmp[1]) );
}
#else
#undef MACH_TIMERS
#endif


/*
  Main function for debugging code generation.
  
  argc - number of command-line parameters
  argv - command-line parameters
  return - 0 on success, non-zero otherwise
*/
int main( int argc, char *argv[] ) {

  unsigned int size, stride;
  char *code;
#ifdef REP_BLOCK
  unsigned int sets = MIN_SETS;
#else
  unsigned int s, sets = MIN_SETS;
  void *ret = &&retaddr;
#endif
#ifdef MACH_TIMERS
  long long start, stop;
#else
  struct timeval start, stop;
#endif
  long nanos, ops;
  
  /* check command line arguments */
  if( argc < 2 ) {
    printf( "usage: dbgCode <size> <stride>\n" );
    return( -2 );
  }
  /* get size and stride from command line */
  size = atoi( argv[1] );
  stride = atoi( argv[2] );

  /* generate enough space for code */
  code = (char *)malloc( (size * 2) * sizeof(char) );
  if( ! code ) {
    printf( "error: couldn't allocate enough space.\n" );
    return( -1 );
  }  
  /* print code block sizes */
  printf( "Block Sizes: jmp=%d nop=%d ret=%d\n", jmpBlockSize(), nopBlockSize(), retBlockSize() );   

  /* generate code and execute */
  generateCode( code, size, stride );
 
#ifdef MACH_TIMERS
  start = rdtsc();
#else
  gettimeofday( &start, NULL );
#endif
 
 run:
#ifdef REP_BLOCK
  REP_BLOCK( code, stride, sets );
#else
  RUN_BLOCK( code, stride, ret );

  printf( "Didn't work... :-(\n" );
  return( 1 );
#endif

  goto retaddr;
  goto run;
  
 retaddr:      
#ifdef MACH_TIMERS
  stop = rdtsc();
  nanos = (((long)(stop - start)) * 2);
#else
  gettimeofday( &stop, NULL );
  nanos = ((stop.tv_sec * MILL + stop.tv_usec) - (start.tv_sec * MILL + start.tv_usec)) * 1000;
#endif

  ops = (stride) ? (size / stride) : (size / nopBlockSize());
  printf( "%d %.4f\n", stride, (double)nanos / (double)(ops * sets) );
  
  printf( "It works.. :-)\n" ); 
  return( 0 );
}
