/*
 *  MOB: A Memory Organization Benchmarking Tool 
 *
 *  Copyright (C) 2000-2001 Josep M. Blanquer and Robert C. Chalmers 
 *
 *  This package is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Josep M. Blanquer <blanquer@cs.ucsb.edu> 
 *          Robert C. Chalmers <robertc@cs.ucsb.edu> 
 *
 */
/**
 * Skeleton definitions of instruction primitives for unsupported architectures.
 **/


#ifndef __MACHDEP_H__
#define __MACHDEP_H__

#define MACH_NONE


/* architecture specific code blocks */

/*
  Instructions to start a loop of executions.  The count is
  the number of sets to execute the code block.  Each of the 
  parameters should be a value on the stack.  The next address
  is stored in AX, the amount to jump is stored in BX.
*/
#define REP_BLOCK( code, jump, count ) 

/* 
   Instructions to start executing generated code on "code", 
   jumping "jump" bytes and returning to "ret".  Each of the 
   parameters should be a value on the stack.  The next address
   is stored in AX, the amount to jump is stored in BX, and the
   return address is stored in DX.
*/
#define RUN_BLOCK( code, jump, ret ) 

/*
  Instructions to jump within the code block to the next jump 
  instruction or the return block.  The jump size is added to 
  the currrent address in AX.  Any delay slots must be filled
  in this block since the compiler will not see it.
*/
#define JMP_BLOCK startjmp: \
                  endjmp:

/*
  Instruction used as filler.
*/
#define NOP_BLOCK  startnop: \
                   endnop:

/*
  Instruction used to return back to the calling code.
  The return address is excepted in DX.  Any delay slots 
  must be filled in this block since the compiler will not 
  see it.
*/
#define RET_BLOCK startret: \
                  endret:


#endif
