/*
 *  MOB: A Memory Organization Benchmarking Tool 
 *
 *  Copyright (C) 2000-2001 Josep M. Blanquer and Robert C. Chalmers 
 *
 *  This package is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Josep M. Blanquer <blanquer@cs.ucsb.edu> 
 *          Robert C. Chalmers <robertc@cs.ucsb.edu> 
 *
 */
/**
 * Zone Operations definition for the mob configuration file sections.
 **/

#ifndef __ZONE_H__
#define __ZONE_H__

/* Maximum of different types of object zones in the system */
#define MAX_ZONE_OBJS  10


#define MAX_LINE_LEN  250 
#define MAX_SYMBOL_LEN  30
#define MAX_VALUE_LEN   MAX_SYMBOL_LEN

/* Operations that every zone object type has to provide */
struct vZoneOps{
  char * name;  /* Zone Object name */
  /* Clears and deallocates the system objects of this zone type */
  void (* clearObjects)(void);
  void *(* createObject)(void); /* creates an dataObject zone specific */
  /* Adds the zone specific pair to the zone data object */
  int (* addPair)( void * obj, char * symbol,char * value );
  /* Checks and stores a completed zone object */
  int (* flushObject)( void * obj);
  /* Prints the object to the specified file pointer stream */
  void (* printObject)( FILE * fp, void * obj);
};

/* The list of registered zones of the system */
#define CACHEOPS {"cache",\
                   cache_clearObjects,\
		   cache_createObject,\
		   cache_addPair,\
		   cache_flushObject,\
		   cache_printObject}

#define TLBOPS {"tlb",\
                 tlb_clearObjects,\
	         tlb_createObject,\
		 tlb_addPair,\
		 tlb_flushObject,\
		 tlb_printObject}

/* It returns the zone operations from a zone name.
   Returns NULL if the zone is not defined in the global list */
struct vZoneOps * getZoneOps( char * zoneName );

/* Zone specific functions */
void   cache_clearObjects();
void * cache_createObject(void);
int    cache_addPair( void * obj, char * symbol,char * value );
int    cache_flushObject( void * obj);
void   cache_printObject( FILE * fp, void * obj);

void   tlb_clearObjects();
void * tlb_createObject(void);
int    tlb_addPair( void * obj, char * symbol,char * value );
int    tlb_flushObject( void * obj);
void   tlb_printObject( FILE * fp, void * obj);

#endif /*__ZONE_H__*/
